#ifndef _CLOCKH_
#define _CLOCKH_


//// clockSDA P50 clockSCL P51
//// alarm P15

//// NOTE:  clock register entries are set as hex data, because clock works with hex values (decimal read!)



#define BUS_READ          0x9F
#define BUS_WRITE         0x9E

#define ACC_CSR           0xAC      // Access Configuration/Control Register
#define ACC_CLOCK         0xC0      // Access Clock Register
#define ACC_CLOCK_ALARM   0xC7      // Access Clock Alarm Register
#define ACC_TH            0xA1      // Access Thermostat Setpoint High
#define ACC_TL            0xA2      // Access Thermostat Setpoint Low
#define ACC_CSRAM         0x17      // Access Clock 32 byte SRAM
#define ACC_RT            0xAA      // Access Read Temperatur Register

#define ADR_SECONDS       0x00  
#define ADR_MINUTES       0x01  
#define ADR_HOURS         0x02  
#define ADR_WEEKDAY       0x03  
#define ADR_DAY           0x04  
#define ADR_MONTH         0x05 
#define ADR_YEAR          0x06  

#define MASK_SECONDS      0x7F      // bits 0-6
#define MASK_MINUTES      0x7F      // bits 0-6
#define MASK_HOURS        0x3F      // bits 0-5
#define MASK_WEEKDAY      0x07      // bits 0-2
#define MASK_DAY          0x3F      // bits 0-5
#define MASK_MONTH        0x1F      // bits 0-5
#define MASK_YEAR         0xFF      // bits 0-7

#define CSR_OS1          0x80      
#define CSR_OS0          0x40      
#define CSR_A1           0x20      
#define CSR_A0           0x10    
#define CSR_CNV          0x04      
#define CSR_POL          0x02     
#define CSR_1SH          0x01      
  
#define MASK_CAF          0x0080      // CAF flag
#define MASK_TAF          0x0040      // TAF flag
#define MASK_CAL          0x0020      // CAL flag
#define MASK_TAL          0x0010      // TAL flag
#define MASK_CAF_TAF      0x00C0      // TAF and CAF flag


void initClock(void);

char loadByteCSRAM(char adr);
void storeByteCSRAM(char adr, char data);

void enableTemperatureAlarm(void);
void disableTemperatureAlarm(void);
void enableClockAlarm(void);
void disableClockAlarm(void);
char isTemperatureAlarmEnabled(void);
char isClockAlarmEnabled(void);

void setTemperatureThresholdHigh(char* tt_high);
void setTemperatureThresholdLow(char* tt_low);

void readTemperature(char* adr);
void readTemperatureThresholdHigh(char* adr);
void readTemperatureThresholdLow(char* adr);

void readDate(char* adr);
void readTime(char* adr);
void readAlarmTime(char* adr);

char clockAlarm(void); 
char temperatureAlarm(void);
void stopClockAlarm(void);  

void setDate(char* date);
void setTime(char* time);
void setAlarmTime(char* ext_time);

void setCSReg(char setting);

void terminal_handleClockAlarmIRQ(void);
int getTemp(void);

int getReg16bit(char acc,int bitmask);
void loadTemperature(char* temp, int value);

// timeRecord functions
struct timeRecord {
    unsigned int tseconds : 3; // 0..5
    unsigned int seconds  : 4; // 0..9
    unsigned int tminutes : 3; // 0..5
    unsigned int minutes  : 4; // 0..9
    unsigned int thours   : 2; // 0..2
    unsigned int hours    : 4; // 0..9
    unsigned int tday     : 2; // 0..3
    unsigned int day      : 4; // 0..9
    unsigned int tmonth   : 1; // 0..1
    unsigned int month    : 4; // 0..9
    unsigned int tyear    : 4; // 0..9
    unsigned int year     : 4; // 0..9
};

void timeRecordToString(struct timeRecord* t, char* c);
void timeRecordFromString(struct timeRecord* t, char* c);
void createTimeRecord(struct timeRecord* t);
void setTimeRecord(struct timeRecord* t);
unsigned char compareTimeStamps(struct timeRecord* t1, struct timeRecord* t2);
long int timeStampDiff(struct timeRecord* t1, struct timeRecord* t2);


#endif
