#include "common.h"
#include "sensors.h"
#include "serial.h"
#include "clock.h"
#include "eeprom.h"
#include "funk.h"
#include "ehandler.h"
#include "sms.h"

// broadcasts the event into the radio
void broadcastEventToRadio(struct sensorDataRecord* event){
  sendPacket(BROADCAST, eepromConfig.id, SENSOR_PACKET, 0x55, sizeof(*event), (char*) event);
}


// prints an event to the serial port
void printEventToSerial(struct sensorDataRecord* event, unsigned char printall){
    char timeString[18];
    char idString[6];
    char temperatureString[7];
    
    idString[ writeUnsignedInt((*event).id, (char*) idString) ] = 0x00;      
    timeRecordToString((struct timeRecord*) &((*event).timeStamp), (char *) timeString);
    loadTemperature((char*) temperatureString, (*event).temperature);
    temperatureString[6] = 0x00;

    //sendRS232('!');
    //sendRS232Int(rxpValue);
    //sendRS232('!');

    sendRS232('[');
    sendRS232String((char*) idString);
    sendRS232String("|");
    sendRS232String((char*) timeString);
    sendRS232String("|");
    sendRS232String((char*) temperatureString);
    sendRS232(']');

    
    if( printall || (*event).sensorFlags & SENSOR_RC5){
      sendRS232String("[IR: C(");
      sendRS232Int(((*event).rc5code & 0x003F));
      sendRS232String(") A(");
      sendRS232Int(((*event).rc5code & 0x07C0) >> 6);
      sendRS232String(")]");
    }                 
    if( printall || (*event).sensorFlags & SENSOR_BUTTON){
      sendRS232String("[Btn: ");
      if(event->sensorFlags & SENSOR_BUTTON) sendRS232('1'); else sendRS232('0'); 
      sendRS232(']');
    }                 
    if( printall || (*event).sensorFlags & SENSOR_LIGHT){
      sendRS232String("[Light: "); sendRS232LongInt((*event).lightCount);
      sendRS232(']');
    }
    if( printall || (*event).sensorFlags & SENSOR_PIR){
      sendRS232String("[Pir: "); sendRS232Int((*event).pirCount);
      sendRS232(']');
    }
    if( printall || (*event).sensorFlags & SENSOR_VIB){
      sendRS232String("[Vib: "); sendRS232Int((*event).vibCount);
      sendRS232(']');
    }
    if( printall || (*event).sensorFlags & SENSOR_MIC){
      sendRS232String("[Mic: "); sendRS232Int((*event).micCount);
      sendRS232(']');
    }
    if( printall || (*event).sensorFlags & SENSOR_BATTERYALARM){
      sendRS232String("[BAT: "); sendRS232Int((*event).batteryValue);
      sendRS232(']');
    }  
    if( printall || (*event).sensorFlags & SENSOR_EXTERNALARM){
      sendRS232String("[EXT: "); sendRS232Int((*event).externValue);
      sendRS232(']');
    }  
  
    sendRS232String("\r\n");   
}

// process an event to eeprom, radio and serial
void processEvent(struct sensorDataRecord* event){
  if(eepromConfig.announceFlags & AF_TO_EEPROM) eeprom_appendEvent(event); 
  if(eepromConfig.announceFlags & AF_TO_RADIO) broadcastEventToRadio(event); 
  if(eepromConfig.announceFlags & AF_TO_SERIAL) printEventToSerial(event, 0);
  if(eepromConfig.announceFlags & AF_HANDY) sendSMSEvent(eepromConfig.smsnr, event); 
}

// check if local events occured
void checkLocalEvents(void){
  // should local events be processed? && new local eventdata available?
  if((eepromConfig.announceFlags & AF_PROC_LOCAL) && (sensorData.sensorFlags)){
    processEvent((struct sensorDataRecord*) &sensorData); // process event
    sensorData.sensorFlags = 0x0000;                      // then clear flags    
  }
}

// handles an event which was received from another sensorboard, called by funkPacketHandler()
void handleSensorPacket(void){
  if(eepromConfig.announceFlags & AF_PROC_RADIO){
    processEvent( (struct sensorDataRecord*) &rxBuf[6] );
  }
}



