#ifndef _SENSORSH_
#define _SENSORSH_

#include "clock.h"


//  masks for the different sensor flags (sensorData.sensorFlags)
#define SENSOR_LIGHT   (0x02)            // these three (LIGHT, PIR, VIB) correspond directly
#define SENSOR_PIR     (0x08)            // to the positions on P1 (used for P1IFG, P1SEL etc..) 
#define SENSOR_VIB     (0x10)            // so never change these!!!
#define SENSOR_BATTERYALARM  (0x01)
#define SENSOR_EXTERNALARM   (0x04)
#define SENSOR_MIC           (0x20)
#define SENSOR_RC5           (0x40)
#define SENSOR_BUTTON        (0x80)

void setFlag(unsigned char flag);

extern volatile unsigned int micAverage; // initialized by initMicrophone()

void sensor_init(void);

volatile struct sensorDataRecord {

  unsigned int id; // id of the sensor this record belongs to

  // private int to keep the updated flags (similar to interrupt flags, set when value is updated)
  // poll these via the valueUpdated method
  unsigned int sensorFlags;
  
  // temporary counters are read out to this variables
  unsigned long int lightCount;  
  unsigned int pirCount;
  unsigned int vibCount;
  unsigned int micCount;
  
  // read out by the ad continously, ADValuesUpdated is set to 1 if updated
  unsigned int micValue;
  unsigned int externValue;
  unsigned int batteryValue;
  
  unsigned int temperature;
  
  unsigned int rc5code;
  
  unsigned int fiveMsTimer;   // counts up in 5ms ticks

  volatile struct timeRecord timeStamp;

} sensorData;



extern volatile unsigned int rxpTemp, rxpValue;
volatile unsigned char newADValues;

#endif
