#include <io.h>
#include <signal.h>

#include "terminal.h"
#include "clock.h"
#include "serial.h"
#include "funk.h"
#include "eeprom.h"
#include "common.h"
#include "sensors.h"
#include "sendir.h"
#include "watchdog.h"
#include "recir.h"
#include "ehandler.h"
#include "sms.h"

// Control the MSP via RS232 with simple ascii commands

#define TERM_BUFSIZE      (50)

volatile char command[TERM_BUFSIZE];            // command buffer, commands are transferred from serbuf to here
volatile char reply[TERM_BUFSIZE];              // reply buffer, most commands build their reply here and use sendReply()
volatile char serbuf[TERM_BUFSIZE];             // the serial IRQ writes bytes into this buffer
          
volatile int cpos=0;                            // pos in receive buffer (serbuf[])
volatile unsigned char serReceivedFlags = 0x00; // control flags for receiving from the serial port
char lastNum[13] = "09999999999";               // number from a SMS is written here and used for replying

// most commands uses this for sending their reply, which they wrote into the reply[] field 
void sendReply(int n) {
  int i;

  if(!(eepromConfig.announceFlags & AF_HANDY)){ // normale Behandlung 
    for(i=0;i<n;i++) sendRS232(reply[i]);
    sendRS232('\n');
  }
  else {                                        // Behandlung fuers Handy
    if(prepareSendSMS(lastNum)){
      sendRS232Chars((char*)&command[0], 3);
      sendRS232String(": ");
      for(i=0;i<n;i++) sendRS232(reply[i]);
      finishSMS();
    }
  }
}

#define COM(a,b,c)  (command[0]==a && command[1]==b && command[2]==c)

// "task entry point"
void terminal_main(void) {
  if(serReceivedFlags & (SER_COMMAND_READY+SER_STREAM_READ)) executeCommand((char*)&command[0]);
}


// executes a command                       
void executeCommand(char* command) {
  if(COM('d','e','a')) {   			            // dea  erases the whole eeprom     
    stopWatchdog();
    unsigned int i;
    for(i=0; i<=0x1FFF; i++){
      writeEEPROMByte((i>>8), i, 0xFF);
      if((i % 0x0100)==0 ) sendRS232('.');      
    }
    reply[0] = 'O'; reply[1] = 'K';
    sendReply(2);
    startWatchdog();
  }
  else if(COM('r','a','t')) {					            // rat  read alarm time
    readAlarmTime((char *) reply);               
		sendReply(10);
	}
  else if(COM('r','b','p')) {                     // rbp  read beeper
		if(P2OUT&0x08) reply[0]='1'; else reply[0]='0';
		sendReply(1); 
	}	
	else if(COM('r','b','s')) {                     // rbs read button
		if(P2OUT&0x80) reply[0]='1'; else reply[0]='0';
		sendReply(1); 
	}	
	else if(COM('r','c','a')) {                     // rca
		if(clockAlarm()) reply[0]='1'; else reply[0]='0';
		sendReply(1); 
	}
	else if(COM('r','c','e')) {                     // rce
		if(isClockAlarmEnabled()) reply[0]='1'; else reply[0]='0';
		sendReply(1); 
	}		
  else if(COM('r','c','t')) {                     // rct
		readTime((char *) reply);
		sendReply(8); 
	}	
  else if(COM('r','d','d')) {                     // rdd
		readDate((char *) reply);
		sendReply(10);
  }
  else if(COM('r','e','b')) {   			            // reb  
    char higha, lowa, eb;
    fromHexToByte((char*) &command[4], &higha );
    fromHexToByte((char*) &command[6], &lowa );
    eb = readEEPROMByte(higha, lowa);  
    fromByteToHex(&eb, (char*) reply);
    sendReply(2);    
  }
	else if(COM('r','e','r')) {   	                // rer
	  stopWatchdog();		             
    unsigned char higha, lowa, eb;
    unsigned int fromad, toad, curad;
    
    fromHexToByte((char*) &command[4], &higha );
    fromHexToByte((char*) &command[6], &lowa );
    fromad = (higha << 8) + lowa;
    
    fromHexToByte((char*) &command[9], &higha );
    fromHexToByte((char*) &command[11], &lowa );
    toad = (higha << 8) + lowa;
    
    for(curad=fromad; (curad <= toad) && (curad <= 0x1FFF) ; curad++){     
      if( ((curad - fromad) % 16) == 0){
        sendRS232String("\r\n");
        fromByteToHex((char*) (&curad)+1, (char*) &reply[0] );
        fromByteToHex((char*) (&curad), (char*) &reply[2]);
        sendRS232(reply[0]);
        sendRS232(reply[1]);
        sendRS232(reply[2]);
        sendRS232(reply[3]);
        sendRS232(':'); 
      }
      
      eb = readEEPROMByte((curad>>8), curad);
      fromByteToHex(&eb, (char*) reply);

      sendRS232(' ');
      sendRS232(reply[0]);
      sendRS232(reply[1]);    
    }
    sendRS232String("\r\n");
    startWatchdog();
  }

  else if(COM('r','l','g')) {
    if(P2OUT&0x02) reply[0]='0'; else reply[0]='1';
    sendReply(1); 
  } 
  else if(COM('r','l','r')) {
    if(P2OUT&0x01) reply[0]='0'; else reply[0]='1';
    sendReply(1); 
  } 
  else if(COM('r','l','y')) {
    if(P2OUT&0x04) reply[0]='0'; else reply[0]='1';
    sendReply(1); 
  }   
  else if(COM('r','l','s')) {                     // rls
		sendReply( writeUnsignedLongIntFixed(sensorData.lightCount, (char*) reply, 6 ));		
  } 
  else if(COM('r','m','m')) {                     // rmm
		sendReply( writeUnsignedIntFixed(sensorData.micValue, (char*) reply, 4 ));		    
  }
  else if(COM('r','m','s')) {                     // rms
  	sendReply( writeUnsignedIntFixed(sensorData.pirCount, (char*) reply, 3 ));		
  } 
  else if(COM('r','m','a')) {                     // rma reads mic average (bjrn)
		sendReply( writeUnsignedIntFixed(micAverage, (char*) reply, 4));
  }
  else if(COM('r','m','i')) {                     // rma reads mic window (bjrn)
		sendReply( writeUnsignedIntFixed(eepromConfig.micInterval, (char*) reply, 4));
  }  
  else if(COM('r','r','p')) {                     // rrp
		sendReply( writeUnsignedIntFixed(rxpTemp, (char*) reply, 4));
	}
  else if(COM('r','t','m')) {                     // rtm
  	sendReply( writeUnsignedIntFixed( getFunkMode(), (char*) reply, 1)); 
  } 
  else if(COM('r','t','p')) {                     // rtp
  	sendReply( writeUnsignedIntFixed( getFunkPower(), (char*) reply, 2)); 
  } 
  else if(COM('r','t','a')) {                     // rta
		if(temperatureAlarm()) reply[0]='1'; else reply[0]='0'; 
		sendReply(1); 
	}
  else if(COM('r','t','e')) {                     // rte
		if(isTemperatureAlarmEnabled()) reply[0]='1'; else reply[0]='0'; 
		sendReply(1); 
	}
	else if(COM ('r', 't', '5')){
    sendReply( writeUnsignedInt(sensorData.fiveMsTimer, (char*) reply ) );
	}	
	else if(COM('r','t','h')) { 					           // rth
		readTemperatureThresholdHigh((char *) reply);
    sendReply(6);
  }
  else if(COM('r','t','l')) {                     // rtl
		readTemperatureThresholdLow( (char *) reply);
    sendReply(6);
  }
	else if(COM('r','t','t')) {                   	 // rtt
		readTemperature((char *) reply);
    sendReply(6);
	}
  else if(COM('r','v','b')) {                   	 // rvb
  	sendReply( writeUnsignedIntFixed( sensorData.batteryValue, (char*) reply, 4)); 
	}
  else if(COM('r','v','e')) {                   	 // rve
  	sendReply( writeUnsignedIntFixed( sensorData.externValue, (char*) reply, 4)); 
	}
  else if(COM('r','v','s')) {                     // rvs
  	sendReply( writeUnsignedIntFixed( sensorData.vibCount, (char*) reply, 3)); 
  } 
  else if(COM('r','m','c')) {                     // rvo (bjrn)
		sendReply( writeUnsignedIntFixed(sensorData.micCount, (char*) reply, 5 ));
  }   
	else if(COM('s','a','t')) { 	                   // sat		
		setAlarmTime((char *) &command[4]); 
	}
  else if(COM('s','b','p')) {                     // sbp
		if(command[4]=='1') beepOn();
		if(command[4]=='0') beepOff();
	}	
	else if(COM('s','c','a')) { 	                   // sca
		stopClockAlarm();
	}	
	else if(COM('s','c','e')) { 	                   // sce
		if(command[4]=='1') enableClockAlarm();
	  if(command[4]=='0') disableClockAlarm();
	}
  else if(COM('s','c','t')) { 			               // sct
		setTime((char *) &command[4]);
	}
	else if(COM('s','d','d')) {                      // sdd					
	 setDate((char *) &command[4]);
	}

  // Leds:  "slr", "slg", "sly"
	else if(COM('s','l','r')) (command[4]=='1' ? redOn() : redOff());
	else if(COM('s','l','g')) (command[4]=='1' ? greenOn() : greenOff());
	else if(COM('s','l','y')) (command[4]=='1' ? yellowOn() : yellowOff());
	 

  else if(COM('s','m','a')) {                     // rma reads mic average (bjrn)
		micAverage = readUnsignedInt((char*) &command[4], 4);
  }
  else if(COM('s','m','i')) {                     // rma reads mic window (bjrn)
    eepromConfig.micInterval = readUnsignedInt((char*) &command[4], 4);
    saveConfig();
  }  

	else if(COM('s','t','e')) {   			             // sta
		if(command[4]=='1') enableTemperatureAlarm();
		if(command[4]=='0') disableTemperatureAlarm();
	}
	else if(COM('s','t','h')) {   					         // sth
		setTemperatureThresholdHigh((char *) &command[4]);
	}
	else if(COM('s','t','l')) {   			             // stl
    setTemperatureThresholdLow((char *) &command[4]);
  }
  else if(COM('s','t','m')) {                     // stm
    setFunkMode(command[4] & 0x0F); 
  } 
  else if(COM('s','t','p')) {                     // stp
    setFunkPower( readUnsignedInt((char*) &command[4], 2) );
    saveConfig();
  } 
  else if(COM('w','e','b')) {   		              // web	             
    char higha, lowa, eb;

    fromHexToByte((char*) &command[4], &higha );
    fromHexToByte((char*) &command[6], &lowa );
    fromHexToByte((char*) &command[9], &eb );

    if(writeEEPROMByte(higha, lowa, eb)){
      reply[0] = 'O'; reply[1] = 'K';
      sendReply(2);
    } else {
      reply[0] = 'F'; reply[1] = 'A'; reply[2] = 'I'; reply[3] = 'L';
      sendReply(4);    
    }    
  }
  else if(COM('w','i','r')) {   		                // wir	             
    unsigned char high, low;
    unsigned int data;
    fromHexToByte((char*) &command[4], &high );
    fromHexToByte((char*) &command[6], &low );
    data = high;
    data = (data << 8) + low;
    sendIRData(data);
    sendRS232Int(data);
    sendRS232String(" gesendet. \n"); 
  }
  
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ Neue (bjrn)...
  // reset the MSP
  else if(COM('R','S','T')) reset();
  // set/read device id
  else if(COM('s','i','d')){ eepromConfig.id = readUnsignedInt((char*) &command[4], 4); saveConfig(); }    
  else if(COM('r','i','d')) sendReply( writeUnsignedIntFixed(eepromConfig.id, (char*) reply, 4));
  // set/read extern limit  
  else if(COM('s','e','l')){ eepromConfig.externLimit = readUnsignedInt((char*) &command[4], 4); saveConfig(); }
  else if(COM('r','e','l')) sendReply( writeUnsignedIntFixed(eepromConfig.externLimit, (char*) reply, 4));
  // set/read battery limit
  else if(COM('s','b','l')){ eepromConfig.batteryLimit = readUnsignedInt((char*) &command[4], 4); saveConfig(); }    
  else if(COM('r','b','l')) sendReply( writeUnsignedIntFixed(eepromConfig.batteryLimit, (char*) reply, 4));
  // set/read lower/upper light limits
  else if(COM('s','l','l')){ eepromConfig.lowerLightLimit = readUnsignedLongInt((char*) &command[4], 6); saveConfig(); }    
  else if(COM('r','l','l')) sendReply( writeUnsignedLongIntFixed(eepromConfig.lowerLightLimit, (char*) reply, 6));
  else if(COM('s','u','l')){ eepromConfig.upperLightLimit = readUnsignedLongInt((char*) &command[4], 6); saveConfig(); }    
  else if(COM('r','u','l')) sendReply( writeUnsignedLongIntFixed(eepromConfig.upperLightLimit, (char*) reply, 6));
  // set funk rxpower limits
  //else if(COM('s','f','a')){ eepromConfig.rxActivityLimit = readUnsignedInt((char*) &command[4], 4); saveConfig(); }    
  //else if(COM('r','f','a')) sendReply( writeUnsignedIntFixed(eepromConfig.rxActivityLimit, (char*) reply, 4));
  else if(COM('s','f','r')){ eepromConfig.rxReceiveLimit = readUnsignedInt((char*) &command[4], 4); saveConfig(); }    
  else if(COM('r','f','r')) sendReply( writeUnsignedIntFixed(eepromConfig.rxReceiveLimit, (char*) reply, 4));
  // set/read announce flags
  else if(COM('s','a','f')){ fromHexToByte((unsigned char*) &command[4], &eepromConfig.announceFlags); saveConfig(); }
  else if(COM('r','a','f')){ fromByteToHex(&eepromConfig.announceFlags, (unsigned char*)reply); sendReply(2); }
  // read ir
  else if(COM('r','c','f')) printConfig();
  else if(COM('r','i','r')){   
    fromByteToHex((char*) (&ir_data)+1, (char*) &reply[0] );
    fromByteToHex((char*) (&ir_data), (char*) &reply[2]);
    sendReply(4);
  }
  else if(COM('s','n','t')){   
    struct timeRecord time;
    char timeString[18];

    timeRecordFromString(&time, (char*) &command[4]);
    broadcastTimeToRadio(&time);
    setTimeRecord(&time);
    timeRecordToString(&time, (char *) timeString);
    sendRS232String((char*) timeString);
    sendRS232String(" OK \n");

  }
  else if(COM('r','e','v')){
    unsigned int i;
    unsigned int s = readUnsignedInt((char*) &command[4], 3);
    unsigned int e = readUnsignedInt((char*) &command[8], 3);
    struct sensorDataRecord event;
    stopWatchdog();
    
    sendRS232String("eventMemPointer: ");
    sendRS232Int(eventMemPointer);
    sendRS232('\n');
    
    for(i=s; i<=e; i++){
      readEEPROM(EVENTMEM_ADDRESS + (i * EVENT_SIZE), (char*) &event, EVENT_SIZE);
      if(event.id == 0xFFFF) break;
      sendRS232Int(i);
      sendRS232String(": ");
      printEventToSerial(&event, 0);      
    }  
  }
  else if(COM('s','m','r')){
    int i = 0;
    
    for(i=0; i<12; i++) eepromConfig.smsnr[i] = command[4+i];
    eepromConfig.smsnr[12] = '\0';
    saveConfig();       
  }
  else if(COM('r','m','r')){
    int i = 0;
    for(i=0; i<12; i++) reply[i] = eepromConfig.smsnr[i];
    sendReply(11);
  }
  else if(COM('r','s','r')){
    if(eepromConfig.announceFlags & AF_HANDY){ 
      if(prepareSendSMS(lastNum)){
        sendRS232Chars((char*)&command[0], 3);
        sendRS232String(": ");
        printEventToSerial((struct sensorDataRecord*) &sensorData, 1);
        finishSMS();
      }
    } else printEventToSerial((struct sensorDataRecord*) &sensorData, 1);
  }
  else {
    if(!(eepromConfig.announceFlags & AF_HANDY)){
      sendRS232String("UNKNOWN COMMAND: ");
      sendRS232Chars(command, 3);
      sendRS232('\n');
      serReceivedFlags &= ~SER_STREAM_READ; // stop stream read of wrong commands
    }
  }
  
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  serReceivedFlags &= ~SER_COMMAND_READY; // clear command
}


void terminal_handleSerialIRQ(void){
  int i;
  unsigned char temp = RXBUF1;
  
  if(temp=='\r') return; // filter out carriage returns
  if(temp=='>'){         // filter out this char and set according FLAG
    serReceivedFlags |= SER_PROMPT_RECEIVED;
    return;
  }

  serReceivedFlags &= ~SER_STREAM_READ; // stop stream read on any key
  
  if(temp=='\n'){ // line is complete 
    if(eepromConfig.announceFlags & AF_HANDY){ // if handymode test for prefix and copy command, filter out handyanswers
      if(serbuf[0]=='O' && serbuf[1]=='K'){ 
        serReceivedFlags |= SER_OK_RECEIVED; // test for OK 
      }
      else if(serbuf[0]=='E' && serbuf[1]=='R' && serbuf[2]=='R' && serbuf[3]=='O' && serbuf[4]=='R'){
        serReceivedFlags |= SER_ERROR_RECEIVED; // test for ERROR
      }
      else if(serbuf[0]=='+' && serbuf[1]=='C' && serbuf[2]=='M' && serbuf[3]=='T' && serbuf[4]==':'){
         // read out senders mobile number
         for(i=0; i<11; i++) lastNum[i+1] = serbuf[i+10];         
      }
      else if(serbuf[0]=='m' && serbuf[1]=='s' && serbuf[2]=='p'){
        for(i=0; i<TERM_BUFSIZE; i++) command[i] = '\0';       // clear buffer
        for(i=4; i<cpos; i++) command[i-4] = serbuf[i];        // copy command without prefix
        serReceivedFlags |= SER_COMMAND_READY;                 // set command ready
      }
    }
    else { // no handy
      for(i=0; i<TERM_BUFSIZE; i++) command[i] = '\0';         // clear buffer 
      for(i=0; i<cpos; i++) command[i] = serbuf[i];            // copy command
      if(command[4]=='s') serReceivedFlags |= SER_STREAM_READ; // set streamread
      serReceivedFlags |= SER_COMMAND_READY;                   // set command ready
    }
    for(i=0; i<TERM_BUFSIZE; i++) serbuf[i] = '\0';            // clear serial buffer
    cpos = 0;                                                  // reset receive buffer
   } // END line complete

   else if(temp=='\b'){ if(cpos>0) cpos--; }  // backspace  
   else {                                     // write byte into buf
      if(cpos==TERM_BUFSIZE) return;          // dont write anymore if buf full
      serbuf[cpos]=temp;
      cpos++;
   }
}

// prints all config data
void printConfig(){
  char af[3], ll[7], ul[7];
  af[2] = '\0';
  ul[6] = '\0';
  ll[6] = '\0';

  fromByteToHex(&eepromConfig.announceFlags, af);

  writeUnsignedLongIntFixed(eepromConfig.upperLightLimit, ul, 6);  
  writeUnsignedLongIntFixed(eepromConfig.lowerLightLimit, ll, 6); 
    
  sendRS232String("Id: "); sendRS232Int(eepromConfig.id);
  sendRS232String("  AF: "); sendRS232String(af);
  sendRS232String("  configsize: "); sendRS232Int(sizeof(eepromConfig));
  sendRS232('\n');
  
  sendRS232String("batteryLimit: "); sendRS232Int(eepromConfig.batteryLimit);
  sendRS232String("  externLimit: "); sendRS232Int(eepromConfig.externLimit);
  sendRS232('\n');
  
  sendRS232String("lowerLightLimit: "); sendRS232String(ll);
  sendRS232String("  upperLightLimit: "); sendRS232String(ul);
  sendRS232('\n');

  sendRS232String("transmitPower: "); sendRS232Int(eepromConfig.transmitPower);  
  //sendRS232String("  rxActivityLimit: "); sendRS232Int(eepromConfig.rxActivityLimit);
  sendRS232String("  rxReceiveLimit: "); sendRS232Int(eepromConfig.rxReceiveLimit);
  sendRS232('\n');
  
  sendRS232String("smsnr: ");
  sendRS232Chars(eepromConfig.smsnr, 11);
  sendRS232('\n');
}


